# -*- coding: utf-8 -*-
"""
*****************************************************************************************
 *                                                                                     *
*   Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier  *
*   selon les termes de la Licence Publique Générale GNU telle que publiée par          *
*   la Free Software Foundation ; soit la version 2 de la Licence, ou                   *
*   (à votre choix) toute version ultérieure.                                           *
 *                                                                                     *
*****************************************************************************************
"""

import os, shutil, tempfile, zipfile
from pathlib import *
import xml.etree.ElementTree as ET
from qgis.core import *
from qgis.PyQt.QtCore import *
from qgis.PyQt.QtWidgets import *
from qgis.PyQt import uic
from osgeo import gdal

from .ModeleListeCouches import ModeleListeCouches
from . import resources


# Initialize Qt resources from file resources.py
def replaceText(node, newText):
    if node.firstChild.nodeType != node.TEXT_NODE:
        raise Exception("node does not contain text")

    node.firstChild.replaceWholeText(newText)


from .resources import *

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'QPackage_dialog_base.ui'))


class LayerInfo:
    def __init__(self, crs_origin):
        self.crs_origin = crs_origin


class QPackageDialog(QDialog, FORM_CLASS):
    new_project_root: None
    copierCouchesTerminee = pyqtSignal()  # Signal pour indiquer la fin de copierCouches

    def __init__(self, iface, parent=None):

        # Liste des variables utilisables dans toute la classe
        self.driver_name = None
        self.driver_map = None
        self.end_copy = None
        self.ordered_layers = None
        self.extensions_fichiers = None
        self.base_project = None
        self.base_project_path = None
        self.base_project_root = None
        self.base_project_name = None
        self.base_project_name_with_ext = None
        self.base_project_name_ext = None
        self.base_project_crs = None
        self.new_project_path = None
        self.new_project_root = None
        self.new_project_name = None
        self.new_project_name_with_ext = None
        self.new_project_name_ext = None
        self.new_project_crs = None
        self.gpkg = None
        self.list_layers = []
        self.list_gpkg_layers = []
        self.gpkg_layer = None
        self.dirNameQgs = None
        self.dirNameQgz = None
        self.dirNameGpkg = None
        self.iface = iface
        """Constructor."""
        super(QPackageDialog, self).__init__(parent)
        # self.crs_origin = None  # Initialise crs_origin à None
        # uic.loadUi(os.path.join(os.path.dirname(__file__), 'QPackage_dialog_base.ui'), self)
        ui_path = os.path.join(os.path.dirname(__file__), 'QPackage_dialog_base.ui')
        uic.loadUi(ui_path, self)
        # self.setupUi(self)

        self.transform_context = QgsCoordinateTransformContext()

    def tr(self, message):
        return QCoreApplication.translate('qgz2gpkgDialog', message)

    def chercherRepertoire(self):
        filename = QFileDialog.getExistingDirectory(
            self,
            caption=self.tr("Select directory to Fling from..."),
            directory=QDir.currentPath()
        )
        if filename:
            self._repertoire.setText(filename)

    def chargerCouches(self):
        # Créez le dictionnaire de mappage des extensions
        # Récupérer le CRS du projet
        crs = QgsProject.instance().crs()
        self.crs_origin = crs.authid()  # Définit crs_origin sur l'instance
        # self._listeprojections.clear()
        data = []
        for layer in QgsProject.instance().mapLayers().values():
            casecocher = QCheckBox(layer.name())
            if layer.type() == QgsMapLayer.VectorLayer:
                casecocher.setChecked(True)
            data.append(casecocher)
        self._tableau.setModel(ModeleListeCouches(data))
        # Réinitialiser la barre de progression après l'opération
        self._progression.setRange(0, 100)
        self._progression.setValue(0)

    def on_copy_clicked(self):
        """Méthode appelée lorsque le bouton _copy est cliqué."""
        # Démarre l'animation de la barre de progression
        self._progression.setRange(0, 0)  # Met la QProgressBar en mode indéterminé

        # Lance la copie des couches
        self.copierCouches()

        # Réinitialise la barre de progression après l'opération
        self._progression.setRange(100)

    def copierCouches(self, nbrecouches=None):
        self.end_copy = 0
        QgsProject.instance().write()

        model = self._tableau.model()
        root = QgsProject.instance().layerTreeRoot()

        self._progression.setValue(0)
        nbrecouches = sum(1 for row in model.getDonnees() if row.isChecked())
        pas = 100 / nbrecouches if nbrecouches > 0 else 0
        progression = 0

        if not self._repertoire.text():
            QMessageBox.critical(self, self.tr("QPackage"), self.tr("You must choose the destination directory"),
                                 QMessageBox.Ok)
            return  # Arrêter l'exécution si le répertoire n'est pas choisi

        #  Utilisation de Path pour les noms et chemins de fichiers
        self.base_project = QgsProject.instance()
        # Chemin absolu mode binaire
        self.base_project_path = Path(self.base_project.fileName())
        QgsMessageLog.logMessage(self.tr(f"self.base_project_path {str(self.base_project_path)}"), level=Qgis.Info)
        # Chemin racine
        self.base_project_root = self.base_project_path.parent
        QgsMessageLog.logMessage(self.tr(f"self.base_project_root {str(self.base_project_root)}"), level=Qgis.Info)
        # Nom seul sans l'extension
        self.base_project_name = self.base_project_path.stem
        QgsMessageLog.logMessage(self.tr(f"self.base_project_name {str(self.base_project_name)}"), level=Qgis.Info)
        # Nom avec l'extention
        self.base_project_name_with_ext = self.base_project_path.name
        QgsMessageLog.logMessage(self.tr(f"self.base_project_name_with_ext {str(self.base_project_name_with_ext)}"),
                                 level=Qgis.Info)
        # Extension avec le .
        self.base_project_name_ext = self.base_project_path.suffix
        QgsMessageLog.logMessage(self.tr(f"self.base_project_name_ext {str(self.base_project_name_ext)}"),
                                 level=Qgis.Info)

        self.new_project = QgsProject()
        self.new_project.clear()
        self.new_project_path = Path(str(self._repertoire.text() + '/' + self._projectname.text() + '.qgz'))
        QgsMessageLog.logMessage(self.tr(f"self.new_project_path {self.new_project_path}"), level=Qgis.Info)
        # Chemin racine
        self.new_project_root = self.new_project_path.parent
        QgsMessageLog.logMessage(self.tr(f"self.new_project_root : {str(self.new_project_root)}"), level=Qgis.Info)
        QgsMessageLog.logMessage(self.tr(f"self.new_project_root sans str() : {self.new_project_root}"),
                                 level=Qgis.Info)
        # Nom seul sans l'extension
        self.new_project_name = self.new_project_path.stem
        QgsMessageLog.logMessage(self.tr(f"self.new_project_name {str(self.new_project_name)}"), level=Qgis.Info)
        # Nom avec l'extention
        self.new_project_name_with_ext = self.new_project_path.name
        QgsMessageLog.logMessage(self.tr(f"self.new_project_name_with_ext {str(self.new_project_name_with_ext)}"),
                                 level=Qgis.Info)
        # Extension avec le .
        self.new_project_name_ext = self.new_project_path.suffix
        QgsMessageLog.logMessage(self.tr(f"self.new_project_name_ext {str(self.new_project_name_ext)}"),
                                 level=Qgis.Info)
        # self.new_project.setFileName(self.new_project_path)

        self.new_project.setFileName(str(self.new_project_path))
        QgsMessageLog.logMessage(self.tr(f"self.new_project.fileName() {self.new_project.fileName()}"), level=Qgis.Info)

        self.base_project_crs = self.base_project.crs()
        self.new_project.setCrs(self.base_project_crs)
        self.new_project_crs = self.new_project.crs()

        self.extensions_fichiers = ('.csv', '.xls', '.xlsx', '.ods', '.vrt')

        # QgsMessageLog.logMessage(self.tr("Starting layers copy"), level=Qgis.Info)

        # Création de la liste des couches cochées
        checked_layers = []
        # Filtrer les couches cochées en utilisant model.getDonnees()
        # Itération sur les lignes du modèle de données
        for row in model.getDonnees():
            #  Sélection des lignes avec cases cochées
            if row.isChecked():
                # Comparaison avec les items de l'arbre des couches du projet
                for name in QgsProject.instance().mapLayers():
                    layer = QgsProject.instance().mapLayer(name)
                    if (layer.name() == row.text()):
                        checked_layers.append(layer)

        # Inverser l'ordre pour le traitement pour obtenir un arbre des couches dans le nouveau projet dans le même ordre
        # Il est nécessaire pour cela d'inverser l'ordre de l'arbre des couches du projet de base
        self.ordered_layers = checked_layers[::-1]
        # Itération sur chaque couche cochée avec lecture du driver ogr à utiliser pour copier la couche dans le nouveau dossier
        for layer in self.ordered_layers:
            chemin = Path(layer.source())
            file_extension = chemin.suffixes[-1].lstrip('.')
            if file_extension == 'vrt':
                self.copy_vrt_file(layer, chemin)
                continue
            else:
                self.driver_name = QgsVectorFileWriter.driverForExtension(file_extension)
            # self.driver_name = self.get_driver_name(file_extension, self.driver_map)
            print(f"Le driver pour l'extension '{file_extension}' est : {self.driver_name}")
            # QgsMessageLog.logMessage(self.tr(f"Processing layer {layer.name()}"), level=Qgis.Info)
            destination_path = os.path.join(self._repertoire.text(), layer.name())
            # Enregistrer le style de la couche source dans un fichier temporaire
            with tempfile.NamedTemporaryFile(delete=False, suffix='.qml') as temp_file:
                layer.saveNamedStyle(temp_file.name)
                temp_file_path = temp_file.name

            if layer.type() == QgsMapLayer.VectorLayer:
                self.copy_vector_layer(layer, temp_file_path)
            elif layer.type() == QgsMapLayer.RasterLayer:
                self.copy_raster_layer(layer, destination_path)
            progression += pas
            self._progression.setValue(int(progression))

        # Utilisez le répertoire source correct
        source_directory = self.base_project_path
        fichiers_a_copier = self.rechercher_fichiers_avec_extensions(source_directory, self.extensions_fichiers)
        QgsMessageLog.logMessage(self.tr(f"Files to copy : {str(fichiers_a_copier)}. *** NEXT ROUTINE TO CHECK ***"),
                                 level=Qgis.Info)
        # *** ROUTINE À CONTROLER ***
        # self.copier_fichiers(fichiers_a_copier, self._repertoire.text())

        noms_sous_dossiers = ['photos', 'Photos', 'photo', 'Photo']
        sous_dossiers_a_copier = self.rechercher_sous_dossiers(source_directory, noms_sous_dossiers)
        QgsMessageLog.logMessage(
            self.tr(f"Directories to copy : {str(fichiers_a_copier)}. *** NEXT ROUTINE TO CHECK ***"), level=Qgis.Info)
        # *** ROUTINE À CONTROLER ***
        # self.copier_sous_dossiers(sous_dossiers_a_copier, self._repertoire.text())

        # *** ROUTINE À CONTROLER ***
        # self.update_layer_paths()

        QgsMessageLog.logMessage(self.tr(f"Saving project {self.new_project_path}"), level=Qgis.Info)
        self.new_project.write(str(self.new_project_path))

        self._progression.setValue(100)
        QgsMessageLog.logMessage(self.tr("Operation completed successfully"), level=Qgis.Info)

        self.copierCouchesTerminee.emit()


    def copy_vrt_file(self, layer, chemin):
        vrt_file_path = str(chemin)

        # Création du chemin du fichier VRT mis à jour
        new_vrt_file_path = Path(self.new_project_root) / chemin.name

        # Ouvrir le fichier VRT avec GDAL pour vérifier s'il s'agit d'un raster ou d'un vecteur
        vrt_dataset = gdal.Open(vrt_file_path)
        if vrt_dataset is not None:
            print("Ouvert en tant que fichier raster.")
            layer_type = "raster"
        else:
            print(f"Ouvrir {vrt_file_path} en tant que fichier vectoriel.")
            vrt_dataset = gdal.OpenEx(vrt_file_path, gdal.OF_VECTOR)
            if vrt_dataset is None:
                print(f"Impossible d'ouvrir le fichier VRT : {vrt_file_path}")
                return
            layer_type = "vector"

        # Analyse du fichier VRT pour identifier les fichiers sources
        tree = ET.parse(vrt_file_path)
        root = tree.getroot()
        vrt_dir = Path(vrt_file_path).parent

        for datasource in root.findall(".//SrcDataSource"):
            src_file_rel = datasource.text
            src_file_abs = (vrt_dir / src_file_rel).resolve()  # Chemin absolu du fichier source
            src_file_name = os.path.basename(src_file_abs)
            new_src_file_path = Path(self.new_project_root) / src_file_name

            # Copier le fichier source vers le répertoire de destination
            if src_file_abs.exists():
                try:
                    shutil.copy(src_file_abs, new_src_file_path)
                    # Mettre à jour le chemin dans le VRT avec le nouveau chemin relatif
                    datasource.text = new_src_file_path.name
                except Exception as e:
                    print(f"Erreur lors de la copie du fichier source {src_file_abs} : {e}")
            else:
                print(f"Fichier source introuvable : {src_file_abs}")

            # Copier les fichiers associés (si c'est une couche vectorielle comme un SHP)
            if src_file_abs.suffix in ['.shp', '.dbf', '.shx']:
                for ext in ['.shp', '.dbf', '.shx', '.prj', '.cpg']:
                    associated_file = src_file_abs.with_suffix(ext)
                    if associated_file.exists():
                        shutil.copy(associated_file, Path(self.new_project_root) / associated_file.name)

        # Sauvegarder le fichier VRT mis à jour
        try:
            tree.write(new_vrt_file_path)
        except Exception as e:
            print(f"Erreur lors de l'enregistrement du fichier VRT mis à jour : {e}")
            return

        # Charger le fichier VRT mis à jour comme couche QGIS
        new_layer = None
        if layer_type == "raster":
            new_layer = QgsRasterLayer(str(new_vrt_file_path), chemin.stem)
        elif layer_type == "vector":
            new_layer = QgsVectorLayer(str(new_vrt_file_path), chemin.stem, "ogr")

        if new_layer is None or not new_layer.isValid():
            print(f"Erreur lors de la création de la couche à partir de : {new_vrt_file_path}")
            return

        # Ajouter la couche au projet QGIS
        self.new_project.addMapLayer(new_layer)

        # Sauvegarder le projet QGIS
        try:
            self.new_project.write()
            print(f"Projet QGIS enregistré avec succès : {self.new_project.fileName()}")
        except Exception as e:
            print(f"Erreur lors de l'enregistrement du projet QGIS : {e}")


    def copy_vector_layer(self, layer, temp_file_path):
        """Copier une couche vectorielle vers le répertoire de destination en remplaçant l'existante si nécessaire et l'ajouter au projet."""

        chemin = Path(layer.source())

        # Construction du chemin absolu
        new_path = os.path.join(self.new_project_root, chemin.name)
        new_path = os.path.abspath(new_path)  # S'assurer que le chemin est absolu

        # Supprimer le fichier existant si présent
        if os.path.exists(new_path):
            os.remove(new_path)
            print(f"Le fichier existant {new_path} a été supprimé.")

        # Supprimer les fichiers auxiliaires (si format Shapefile ou autre)
        base_name = os.path.splitext(new_path)[0]
        for ext in ['.shx', '.dbf', '.prj', '.cpg', '.qpj']:
            aux_file = base_name + ext
            if os.path.exists(aux_file):
                os.remove(aux_file)
                print(f"Le fichier auxiliaire existant {aux_file} a été supprimé.")

        # Vérifier si le fichier source existe
        if not os.path.exists(layer.source()):
            self.show_warning_popup(f"Le fichier source n'existe pas: {layer.source()}")
            return

        # Configurer les options de sauvegarde
        options = QgsVectorFileWriter.SaveVectorOptions()

        options.driverName = self.driver_name
        options.fileEncoding = "UTF-8"
        options.layerOptions = ["ENCODING=UTF-8"]

        # Contexte de transformation des coordonnées basé sur le CRS de la couche source
        context = QgsCoordinateTransformContext()

        # Copier la couche avec writeAsVectorFormatV3
        error = QgsVectorFileWriter.writeAsVectorFormatV3(
            layer,
            new_path,
            context,
            options
        )

        if error[0] != QgsVectorFileWriter.NoError:
            self.show_warning_popup(
                f"Erreur lors de l'écriture du fichier de {layer.source()} à {new_path} : {error[1]}")
            return
        else:
            print(f"Fichier créé avec succès à : {new_path}")

        # Vérifier si le fichier a été créé
        if not os.path.exists(new_path):
            self.show_warning_popup(f"Le fichier {new_path} n'a pas été créé.")
            return

        # Charger la nouvelle couche
        new_layer = QgsVectorLayer(new_path, layer.name(), "ogr")
        if not new_layer.isValid():
            self.show_warning_popup(f"Erreur lors du chargement de la nouvelle couche depuis {new_path}")
        else:
            # Appliquer le style avant d'ajouter la couche au projet
            self.apply_style_from_source(layer, new_layer, temp_file_path)

            # Ajouter la nouvelle couche au projet self.new_project
            self.new_project.addMapLayer(new_layer)
            print(f"Nouvelle couche chargée et ajoutée au projet depuis : {new_path}")

        QgsMessageLog.logMessage(self.tr(f"Vector layer {new_layer.name()} is copied"), level=Qgis.Info)


    def copy_raster_layer(self, layer, destination_path):
        """Copier une couche raster vers le répertoire de destination."""
        raster_path = layer.publicSource()
        # Test existence du dossier
        if not os.path.exists(raster_path):
            self.show_warning_popup(raster_path)
            return

        file_size = os.path.getsize(raster_path)

        destination_full_path = os.path.normpath(
            os.path.join(self._repertoire.text(), layer.name() + os.path.splitext(raster_path)[1]))
        if os.path.normpath(raster_path) == destination_full_path:
            QgsMessageLog.logMessage(self.tr(f"Source and destination for {layer.name()} are the same, skipping copy."),
                                     level=Qgis.Warning)
        else:
            if file_size > 1e9:
                QMessageBox.warning(self, self.tr("QPackage"),
                                    self.tr(
                                        f"The file {os.path.basename(raster_path)} is large and may take a long time to copy."))

            shutil.copy2(raster_path, destination_full_path)

            new_layer = QgsRasterLayer(destination_full_path, layer.name())
            if new_layer.isValid():
                self.new_project.addMapLayer(new_layer)
            else:
                QgsMessageLog.logMessage(self.tr(f"Raster layer {layer.name()} is not valid"), level=Qgis.Warning)


    def update_layer_paths(self):
        """Mettre à jour les chemins des fichiers SVG, UI, et PY dans les propriétés des couches."""
        for layer in self.new_project.mapLayers().values():
            if isinstance(layer, QgsVectorLayer):
                self.update_svg_paths(layer, self._repertoire.text())
                self.update_ui_py_paths(layer, self._repertoire.text())

    def apply_style_from_source(self, layer, new_layer, temp_file_path):
        """Appliquer le style d'une couche source à une nouvelle couche, en mettant à jour les chemins des fichiers SVG. Reste à traiter fichiers UI, et PY."""
        # Copier les fichiers SVG et mettre à jour les chemins dans le fichier QML
        self.copy_svg_files(temp_file_path, self._repertoire.text())
        # *** ROUTINE À CORRIGER ***
        # Copier les fichiers ui et py et mettre à jour les chemins dans le fichier QML
        # self.copy_ui_py_files(temp_file_path, self._repertoire.text())

        # Appliquer le style mis à jour à la nouvelle couche
        updated_qml_path = temp_file_path.replace('.qml', '_updated.qml')
        if new_layer.loadNamedStyle(updated_qml_path):
            QgsMessageLog.logMessage(self.tr(f"Style successfully applied to {new_layer.name()}"), level=Qgis.Info)
        else:
            QgsMessageLog.logMessage(self.tr(f"Failed to apply style to {new_layer.name()}"), level=Qgis.Warning)

        # Nettoyer les fichiers temporaires
        os.remove(temp_file_path)
        os.remove(updated_qml_path)

    def copy_csv_files_from_xml(self, qml_path, destination_folder, layer):
        # Charger et analyser le fichier XML
        tree = ET.parse(qml_path)
        root = tree.getroot()
        layer_full_name = layer.source()

        # Rechercher toutes les balises <Option> dans les balises <editWidget>
        option_elements = root.findall(".//editWidget//config//Option")

        for option in option_elements:
            # Vérifier si l'attribut 'name' est 'LayerSource'
            if option.attrib.get('name') == 'LayerSource':
                # Extraire la valeur de l'attribut 'value'
                csv_path = option.attrib.get('value')
                bin_csv_path = Path(csv_path)
                # Ajouter un message de débogage pour afficher le chemin du fichier
                message = self.tr(f"csv_path : {csv_path}")
                QgsMessageLog.logMessage(message, level=Qgis.Info)
                message = self.tr(f"bin_csv_path : {str(bin_csv_path)}")
                QgsMessageLog.logMessage(message, level=Qgis.Info)

                # Vérifier que csv_path est une chaîne de caractères non vide
                if isinstance(csv_path, str) and csv_path:
                    # Extraire l'extention
                    extension = bin_csv_path.suffix
                    QgsMessageLog.logMessage(f"extension : {extension}", level=Qgis.Info)
                    if extension in self.extensions_fichiers:
                        if extension == '.csv':
                            message = self.tr(f"Fichier à copier : {csv_path}")
                            QgsMessageLog.logMessage(message, level=Qgis.Info)

                            # Copier le fichier CSV vers le dossier de destination
                            csv_file_name = os.path.basename(csv_path)
                            new_csv_path = os.path.join(destination_folder, csv_file_name)
                            shutil.copy2(csv_path, new_csv_path)
                            message = self.tr(f"new_csv_path : {str(new_csv_path)}")
                            QgsMessageLog.logMessage(message, level=Qgis.Info)

                            # Mettre à jour le chemin du fichier dans le fichier XML
                            option.attrib['value'] = new_csv_path.replace(os.path.sep, '/')
                            message = self.tr(
                                f"new_csv_path avec replace : {str(new_csv_path.replace(os.path.sep, '/'))}")
                            QgsMessageLog.logMessage(message, level=Qgis.Info)

                            # Ajouter la source de texte délimité (csv séparateur "," sans géométrie au nouveau projet QGIS
                            layer = QgsVectorLayer(f"file:///{new_csv_path}?delimiter=,&geomType=none",
                                                   bin_csv_path.stem, "delimitedtext")

                            if not layer.isValid():
                                QgsMessageLog.logMessage(self.tr(f"Layer creation failed for {new_csv_path}"),
                                                         level=Qgis.Warning)
                                continue

                            # Ajouter le fichier csv au nouveau projet
                            # self.new_project.addMapLayer(layer)

                            message = self.tr(f"Added CSV layer to the project: {self.new_project.fileName()}")
                            QgsMessageLog.logMessage(message, level=Qgis.Info)
                            self.modify_path_in_qgz_file(new_csv_path)
                        else:
                            message = self.tr(f"Le fichier ne se termine pas par .csv : {csv_path}")
                            QgsMessageLog.logMessage(message, level=Qgis.Warning)
                else:
                    message = self.tr("Le chemin du fichier dans 'LayerSource' est vide ou mal formé.")
                    QgsMessageLog.logMessage(message, level=Qgis.Warning)

        # # Modification du chemin
        # options = root.findall(".//Option[@name='LayerSource']")
        # # Parcours des éléments trouvés pour mettre à jour le chemin
        # for option in options:
        #     ancien_chemin = option.get('value')
        #     # ancien_chemin = self.uri_source
        #     message = self.tr(f"Chemin à remplacer : {ancien_chemin}. Remplacer par {new_csv_path} si {ancien_chemin} == {self.uri_source}")
        #     QgsMessageLog.logMessage(message, level=Qgis.Info)
        #     if ancien_chemin == self.uri_source:
        #         option.set('value', new_csv_path)
        #         print(f"Chemin remplacé : {ancien_chemin} par {new_csv_path}")

        # Sauvegarder les modifications dans le fichier QML

        # Enregistrer le fichier XML mis à jour
        updated_qml_path = qml_path.replace('.qml', '_updated.qml')
        tree.write(updated_qml_path)
        # Ajouter le fichier csv au nouveau projet
        self.new_project.addMapLayer(layer)

    def copy_svg_files(self, qml_path, new_project_dir):
        """Copier les fichiers SVG référencés dans un fichier QML de style vers un sous-dossier 'symbols' dans le répertoire du nouveau projet."""
        tree = ET.parse(qml_path)
        root = tree.getroot()

        # Créer le sous-dossier symbols
        symbols_dir = os.path.join(new_project_dir, 'symbols')
        os.makedirs(symbols_dir, exist_ok=True)

        # Rechercher les balises <Option> ayant l'attribut name="name" et vérifiant si la valeur se termine par .svg
        option_elements = root.findall(".//Option[@name='name']")
        for option in option_elements:
            svg_path = option.attrib.get('value')
            if svg_path and svg_path.lower().endswith('.svg') and os.path.exists(svg_path):
                # Copier le fichier SVG vers le sous-dossier symbols
                svg_file_name = os.path.basename(svg_path)
                new_svg_path = os.path.join(symbols_dir, svg_file_name)
                shutil.copy2(svg_path, new_svg_path)

                # Mettre à jour le chemin du fichier dans le fichier QML
                option.set('value', new_svg_path.replace(os.path.sep, '/'))

        # Enregistrer le fichier QML mis à jour
        updated_qml_path = qml_path.replace('.qml', '_updated.qml')
        tree.write(updated_qml_path)

    def copy_ui_py_files(self, qml_path, new_project_dir):
        """Copier les fichiers UI et PY d'un fichier QML de style vers les sous-dossiers 'forms' et 'pyfiles' dans le répertoire du nouveau projet."""
        tree = ET.parse(qml_path)
        root = tree.getroot()

        # Créer les sous-dossiers forms et pyfiles
        forms_dir = os.path.join(new_project_dir, 'forms')
        pyfiles_dir = os.path.join(new_project_dir, 'pyfiles')
        os.makedirs(forms_dir, exist_ok=True)
        os.makedirs(pyfiles_dir, exist_ok=True)

        # Trouver les balises contenant les chemins des fichiers UI et PY
        ui_files = root.findall(".//editform")
        py_files = root.findall(".//editforminitfilepath")

        # Copier les fichiers UI vers le sous-dossier forms
        for ui in ui_files:
            ui_path = ui.text
            if ui_path and os.path.exists(ui_path):
                ui_file_name = os.path.basename(ui_path)
                new_ui_path = os.path.join(forms_dir, ui_file_name)
                shutil.copy2(ui_path, new_ui_path)
                ui.text = new_ui_path.replace(os.path.sep, '/')
            elif ui_path:
                self.show_warning_popup(ui_path)

        # Copier les fichiers PY vers le sous-dossier pyfiles
        for py in py_files:
            py_path = py.text
            if py_path and os.path.exists(py_path):
                py_file_name = os.path.basename(py_path)
                new_py_path = os.path.join(pyfiles_dir, py_file_name)
                shutil.copy2(py_path, new_py_path)
                py.text = new_py_path.replace(os.path.sep, '/')
            elif py_path:
                self.show_warning_popup(py_path)

        # Enregistrer le fichier QML mis à jour
        updated_qml_path = qml_path.replace('.qml', '_updated.qml')
        tree.write(updated_qml_path)


    # # Fonction pour copier la structure de l'arbre des couches
    # def copierArbreDesCouches(src_group, dest_group):
    #     """Recopie la structure d'un groupe d'arbres des couches d'un projet à un autre, sans doublons."""
    #     for child in src_group.children():
    #         if isinstance(child, QgsLayerTreeGroup):
    #             # Vérifier si le groupe existe déjà
    #             existing_group = dest_group.findGroup(child.name())
    #             if not existing_group:
    #                 # Créer un nouveau groupe dans le projet de destination s'il n'existe pas
    #                 new_group = dest_group.addGroup(child.name())
    #             else:
    #                 new_group = existing_group
    #             # Copier récursivement le sous-groupe
    #             copierArbreDesCouches(child, new_group)
    #         elif isinstance(child, QgsLayerTreeLayer):
    #             # Vérifier si la couche existe déjà dans le groupe cible
    #             layer = child.layer()
    #             if layer in checked_layers and not dest_group.findLayer(layer.id()):
    #                 # Ajouter la couche au nouveau projet si elle n'est pas déjà ajoutée
    #                 dest_group.addLayer(layer)
    #                 # Retirer la couche de checked_layers pour éviter les doublons
    #                 checked_layers.remove(layer)
    #
    # # Copie de la structure de l'arbre des couches du projet de base vers le nouveau projet
    # copierArbreDesCouches(self.base_project.layerTreeRoot(), self.new_project.layerTreeRoot())


    def update_svg_paths(self, layer, new_project_dir):
        """Mettre à jour les chemins des fichiers SVG dans les propriétés des couches."""
        symbols_dir = os.path.join(new_project_dir, 'symbols')
        symbols = layer.renderer().symbols(QgsRenderContext())
        for symbol in symbols:
            for symbol_layer in symbol.symbolLayers():
                if isinstance(symbol_layer, QgsSvgMarkerSymbolLayer):
                    svg_path = symbol_layer.path()
                    if os.path.exists(svg_path):
                        svg_file_name = os.path.basename(svg_path)
                        new_svg_path = os.path.join(symbols_dir, svg_file_name)
                        shutil.copy2(svg_path, new_svg_path)
                        symbol_layer.setPath(new_svg_path.replace(os.path.sep, '/'))
                    else:
                        self.show_warning_popup(svg_path)

    def update_ui_py_paths(self, layer, new_project_dir):
        """Mettre à jour les chemins des fichiers UI et PY dans les propriétés des couches."""
        if layer.type() == QgsMapLayer.VectorLayer:
            # Accéder directement aux propriétés personnalisées de la couche
            ui_path = layer.customProperty('ui_path', '')
            py_path = layer.customProperty('py_path', '')

            # Mettre à jour les chemins des fichiers .ui et .py
            new_ui_path = self.update_path_in_url(ui_path, new_project_dir, '.ui', 'forms')
            new_py_path = self.update_path_in_url(py_path, new_project_dir, '.py', 'pyfiles')

            # Définir de nouvelles valeurs pour les propriétés personnalisées
            layer.setCustomProperty('ui_path', new_ui_path)
            layer.setCustomProperty('py_path', new_py_path)

            QgsMessageLog.logMessage(
                self.tr(f"Updated UI/PY paths for {layer.name()}: UI={new_ui_path}, PY={new_py_path}"),
                level=Qgis.Info
            )

    def update_path_in_url(self, original_path, new_project_dir, extension, subdir):
        """Mettre à jour un chemin de fichier dans une URL vers un sous-dossier spécifique dans le nouveau répertoire."""
        if original_path and os.path.exists(original_path):
            new_path = os.path.join(new_project_dir, subdir, os.path.basename(original_path))
            shutil.copy2(original_path, new_path)
            return new_path
        elif original_path:
            self.show_warning_popup(original_path)
        return ''

    def show_warning_popup(self, missing_path):
        """Afficher une fenêtre d'alerte pour les chemins manquants."""
        message = self.tr(
            f"{missing_path}' was not found. Please check the file path or the drive letter.")
        # Inscription du fichier non trouvé dans le journal :
        QgsMessageLog.logMessage(message, level=Qgis.Info)
        # Fenêtre surgissante remplacée :
        # QMessageBox.warning(self, self.tr("QPackage"), message)

    def rechercher_fichiers_avec_extensions(self, dossier_source, extensions):
        """Recherche des fichiers avec des extensions spécifiques dans le dossier source."""
        fichiers_a_copier = []
        # for fichier in os.listdir(dossier_source):
        #     if fichier.lower().endswith(tuple(ext.lower() for ext in extensions)):
        #         fichiers_a_copier.append(fichier)
        for root, _, files in os.walk(dossier_source):
            for file_name in files:
                if file_name.lower().endswith(extensions):
                    fichiers_a_copier.append(os.path.join(root, file_name))
        return fichiers_a_copier

    def copier_fichiers(self, fichiers_a_copier, dossier_cible):
        """Copie les fichiers spécifiés dans le dossier cible."""
        for fichier in fichiers_a_copier:
            destination = os.path.join(dossier_cible, os.path.basename(fichier))
            shutil.copy2(fichier, destination)
            QgsMessageLog.logMessage(self.tr(f"Fichier copié : {fichier} -> {destination}"), level=Qgis.Info)

    def rechercher_sous_dossiers(self, dossier_source, noms_sous_dossiers):
        """Recherche des sous-dossiers spécifiques dans le dossier source."""
        sous_dossiers_a_copier = []
        for sous_dossier in noms_sous_dossiers:
            chemin_sous_dossier = os.path.join(dossier_source, sous_dossier)
            if os.path.isdir(chemin_sous_dossier):
                sous_dossiers_a_copier.append(chemin_sous_dossier)
        return sous_dossiers_a_copier

    def copier_sous_dossiers(self, sous_dossiers, destination):
        """Copie les sous-dossiers dans le répertoire de destination."""
        for sous_dossier in sous_dossiers:
            destination_sous_dossier = os.path.join(destination, os.path.basename(sous_dossier))
            if os.path.exists(destination_sous_dossier):
                shutil.rmtree(destination_sous_dossier)
            shutil.copytree(sous_dossier, destination_sous_dossier)

    def copy_and_link_spreadsheet_file(self, layer, temp_file_path):
        # destination_folder = self.new_project_root
        # Nom de la couche à copier et du projet cible
        nom_couche_source = layer.name()
        nom_couche_cible = layer.name()
        # chemin_fichier_source = layer.source()  # Chemin vers le fichier VRT ou ODS

        # Vérifiez si la couche source est valide
        if not layer.isValid():
            QgsMessageLog.logMessage(self.tr(f"Couche {layer.name()} non valide."), level=Qgis.Info)
            self.show_warning_popup(nom_couche_source)
            return

        # Exporter les jointures attributaires
        jointures_info = []
        for join in layer.vectorJoins():
            join_info = {
                'joinLayerId': join.joinLayerId(),
                'joinFieldName': join.joinFieldName(),
                'targetFieldName': join.targetFieldName()
            }
            jointures_info.append(join_info)

        # Copier la définition de la couche
        self.uri_source = layer.dataProvider().dataSourceUri()
        QgsMessageLog.logMessage(self.tr(f"URI de la couche source : {self.uri_source}"), level=Qgis.Info)

        # Créer la couche dans le projet cible
        layer_cible = QgsVectorLayer(self.uri_source, nom_couche_cible, 'ogr')

        # Vérifiez si la couche est valide
        if not layer_cible.isValid():
            raise ValueError("La couche cible n'a pas pu être chargée.")

        # Ajouter la couche au projet QGIS cible
        self.new_project.addMapLayer(layer_cible)
        # QgsMessageLog.logMessage(self.tr(f"La couche {layer_cible.name()} a été ajoutée avec succès"), level=Qgis.Info)

        # Ajouter les jointures à la couche cible
        for join_info in jointures_info:
            join_layer = QgsProject.instance().mapLayer(join_info['joinLayerId'])
            if join_layer and join_layer.isValid():
                join = QgsVectorLayerJoinInfo()
                join.joinLayerId = join_layer.id()
                join.joinFieldName = join_info['joinFieldName']
                join.targetFieldName = join_info['targetFieldName']
                layer_cible.dataProvider().addJoin(join)
            else:
                QgsMessageLog.logMessage(
                    self.tr(f"Erreur: La couche de jointure {join_info['joinLayerId']} n'est pas valide."),
                    level=Qgis.Info)

        QgsMessageLog.logMessage(self.tr("Les jointures ont été recréées."), level=Qgis.Info)

    def modify_path_in_qgz_file(self, new_csv_path):
        # Répertoire temporaire pour l'extraction
        temp_dir = 'temp_extraction'
        # Créer le répertoire d'extraction s'il n'existe pas
        os.makedirs(temp_dir, exist_ok=True)
        # Extraire le contenu du fichier .qgz
        with zipfile.ZipFile(self.new_project_path, 'r') as zip_ref:
            zip_ref.extractall(temp_dir)
        qgs_file_path = self.new_project_path.stem + '.qgs'
        # Charger le fichier .qgs en tant qu'arbre XML
        tree = ET.parse(qgs_file_path)
        root = tree.getroot()
        # Modification du chemin
        options = root.findall(".//Option[@name='LayerSource']")
        # Parcours des éléments trouvés pour mettre à jour le chemin
        for option in options:
            ancien_chemin = option.get('value')
            # ancien_chemin = self.uri_source
            message = self.tr(
                f"Chemin à remplacer : {ancien_chemin}. Remplacer par {new_csv_path} si {ancien_chemin} == {self.uri_source}")
            QgsMessageLog.logMessage(message, level=Qgis.Info)
            if ancien_chemin == self.uri_source:
                option.set('value', new_csv_path)
                print(f"Chemin remplacé : {ancien_chemin} par {new_csv_path}")
        # Recompresser le répertoire d'extraction en un fichier .qgz
        with zipfile.ZipFile(self.new_project_path, 'w') as zip_ref:
            for foldername, subfolders, filenames in os.walk(temp_dir):
                for filename in filenames:
                    file_path = os.path.join(foldername, filename)
                    zip_ref.write(file_path, os.path.relpath(file_path, temp_dir))

        # Supprimer le répertoire temporaire
        shutil.rmtree(temp_dir)
